<?php

/*
 * Sudalys OAuth2 Provider
 */

namespace Sudalys\OAuth2\Client\Provider;

use League\OAuth2\Client\Provider\ResourceOwnerInterface;
use League\OAuth2\Client\Token\AccessToken;

class SudalysResourceOwner implements ResourceOwnerInterface
{
    const PATH_API = '/api/v4/';

    /** @var array */
    private $data;

    /** @var string */
    private $domain;

    /** @var AccessToken */
    private $token;

    /**
     * Creates new resource owner.
     */
    public function __construct(array $response, AccessToken $token)
    {
        $this->data = $response;
        $this->token = $token;
    }

    /**
     * Returns the identifier of the authorized resource owner.
     */
    public function getId(): int
    {
        return (int) $this->get('id');
    }


    public function getDomain(): string
    {
        return $this->domain;
    }

    /**
     * Retrieves ful token resource owner.
     *
     * @return string|null
     */
    public function getIdToken()
    {
        return $this->token;
    }


    /**
     * @return $this
     */
    public function setDomain(string $domain): self
    {
        $this->domain = $domain;

        return $this;
    }

    /**
     * The full name of the owner.
     */
    public function getName(): string
    {
        return $this->get('name');
    }

    /**
     * Username of the owner.
     */
    public function getUsername(): string
    {
        return $this->get('username');
    }

    /**
     * Email address of the owner.
     */
    public function getEmail(): string
    {
        return $this->get('email');
    }

    /**
     * URL to the user's avatar.
     *
     * @return string|null
     */
    public function getAvatarUrl(): string
    {
        return $this->get('avatar_url');
    }

    /**
     * URL to the user's profile page.
     */
    public function getProfileUrl(): string
    {
        return $this->get('web_url');
    }

    public function getToken(): AccessToken
    {
        return $this->token;
    }

    /**
     * Whether the user is active.
     */
    public function isActive(): bool
    {
        return 'active' === $this->get('state');
    }

    /**
     * Whether the user is an admin.
     */
    public function isAdmin(): bool
    {
        return (bool) $this->get('is_admin', false);
    }

    /**
     * Whether the user is external.
     */
    public function isExternal(): bool
    {
        return (bool) $this->get('external', true);
    }

    /**
     * Return all of the owner details available as an array.
     */
    public function toArray(): array
    {
        return $this->data;
    }

    /**
     * @param  mixed|null $default
     * @return mixed|null
     */
    protected function get(string $key, $default = null)
    {
        return isset($this->data[$key]) ? $this->data[$key] : $default;
    }
}
