<?php

namespace App\Controller;

use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Attribute\Route;
use Symfony\Component\HttpClient\HttpClient;
use Symfony\Contracts\HttpClient\Exception\TransportExceptionInterface;
use GuzzleHttp\Client;
use GuzzleHttp\Exception\RequestException;

class ClientTestController extends AbstractController
{
    #[Route('/client/test', name: 'app_client_test')]
    public function index(): Response
    {
        // Get OAuth configuration
        $clientId = $_ENV['SSO_CLIENT_ID'] ?? 'not set';
        $clientSecret = $_ENV['SSO_CLIENT_SECRET'] ?? 'not set';
        $authorizeUrl = $_ENV['SSO_AUTHORIZE_URL'] ?? 'not set';
        $tokenUrl = $_ENV['SSO_TOKEN_URL'] ?? 'not set';
        $userInfoUrl = $_ENV['SSO_USERINFO_URL'] ?? 'not set';
        
        // Test 1: Check if URLs are accessible
        $urlTests = [];
        $client = HttpClient::create(['verify_peer' => false, 'verify_host' => false]);
        
        foreach ([$authorizeUrl, $tokenUrl, $userInfoUrl] as $url) {
            try {
                $response = $client->request('GET', $url, ['timeout' => 3]);
                $status = $response->getStatusCode();
                $urlTests[$url] = "Status: $status";
            } catch (TransportExceptionInterface $e) {
                $urlTests[$url] = "Error: " . $e->getMessage();
            } catch (\Exception $e) {
                $urlTests[$url] = "Exception: " . $e->getMessage();
            }
        }
        
        // Test 2: Try to get a token with client credentials
        $tokenTest = "Not tested";
        try {
            $guzzleClient = new Client([
                'verify' => false,
                'timeout' => 5,
            ]);
            
            $response = $guzzleClient->post($tokenUrl, [
                'form_params' => [
                    'grant_type' => 'client_credentials',
                    'client_id' => $clientId,
                    'client_secret' => $clientSecret,
                ],
                'headers' => [
                    'Accept' => 'application/json',
                ],
            ]);
            
            $tokenTest = $response->getBody()->getContents();
        } catch (RequestException $e) {
            $response = $e->getResponse();
            if ($response) {
                $tokenTest = "Error: " . $response->getStatusCode() . " - " . $response->getBody()->getContents();
            } else {
                $tokenTest = "Error: " . $e->getMessage();
            }
        } catch (\Exception $e) {
            $tokenTest = "Exception: " . $e->getMessage();
        }
        
        return $this->render('client_test/index.html.twig', [
            'client_id' => $clientId,
            'client_secret' => substr($clientSecret, 0, 3) . '***',
            'authorize_url' => $authorizeUrl,
            'token_url' => $tokenUrl,
            'userinfo_url' => $userInfoUrl,
            'url_tests' => $urlTests,
            'token_test' => $tokenTest,
        ]);
    }
} 