<?php

namespace App\Security;

use KnpU\OAuth2ClientBundle\Client\ClientRegistry;
use KnpU\OAuth2ClientBundle\Security\Authenticator\OAuth2Authenticator;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\RouterInterface;
use Symfony\Component\Security\Core\Authentication\Token\TokenInterface;
use Symfony\Component\Security\Core\Exception\AuthenticationException;
use Symfony\Component\Security\Http\Authenticator\Passport\Badge\UserBadge;
use Symfony\Component\Security\Http\Authenticator\Passport\Passport;
use Symfony\Component\Security\Http\Authenticator\Passport\SelfValidatingPassport;
use Symfony\Component\Security\Http\EntryPoint\AuthenticationEntryPointInterface;
use League\OAuth2\Client\Provider\ResourceOwnerInterface;
use Psr\Log\LoggerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;

class OAuthAuthenticator extends OAuth2Authenticator implements AuthenticationEntryPointInterface
{
    private $clientRegistry;
    private $router;
    private $logger;

    public function __construct(ClientRegistry $clientRegistry, RouterInterface $router, LoggerInterface $logger)
    {
        $this->clientRegistry = $clientRegistry;
        $this->router = $router;
        $this->logger = $logger;
    }

    public function supports(Request $request): ?bool
    {
        return $request->attributes->get('_route') === 'connect_sso_check' && $request->query->has('code');
    }

    public function authenticate(Request $request): Passport
    {
        $client = $this->getSudalysClient();
        $accessToken = $this->fetchAccessToken($client);

        return new SelfValidatingPassport(
            new UserBadge($accessToken->getToken(), function () use ($accessToken, $client) {
                $sudalysSsoUser = $this->getSudalysClient()->fetchUserFromToken($accessToken);

                $email = $sudalysSsoUser->getEmail();

                /*
                * On regarde si l'utilisateur est deja connecté
                */
                $connectedUser = $this->em->getRepository(User::class)->findOneBy(['ssoToken' => $sudalysSsoUser->getIdToken()]);
                if ($connectedUser) {
                    return $connectedUser;
                }

                // On reggarde si user existe en bdd
                /** @var User $userInDatabase */
               $user = $this->em->getRepository(User::class)->findOneBy(['email' => $email]); 


                /**
                 * A commenter si on ne veut pas creer l utilisateur s il n existe pas en bdd locale
                 * **/
                if (!$user) {
                    $user = new User();
                    $user->setEmail($sudalysSsoUser->getEmail());
                    $user->setPassword("createAPasswordMethod");
                    $this->em->persist($user);
                }

                // On met a jour le token
                $user->setSsoToken($sudalysSsoUser->getIdToken());
                $this->em->flush();

                return $user;
            })
        );
    }
    
    private function getUserIdentifier(ResourceOwnerInterface $resourceOwner): string
    {
        // Get the unique identifier from the resource owner
        // This depends on your SSO server's response format
        // For standard OAuth servers, this might be the 'sub' field
        $data = $resourceOwner->toArray();
        
        $this->logger->debug('Resource owner data', [
            'data' => $data
        ]);
        
        // Try to get a unique identifier from the data
        if (isset($data['sub'])) {
            return $data['sub'];
        } elseif (isset($data['id'])) {
            return $data['id'];
        } elseif (isset($data['email'])) {
            return $data['email'];
        }
        
        // Fallback to a hash of the entire data if no good identifier is found
        return md5(json_encode($data));
    }

    public function onAuthenticationSuccess(Request $request, TokenInterface $token, string $firewallName): ?Response
    {
        $this->logger->info('OAuth authentication successful', [
            'user' => $token->getUserIdentifier()
        ]);
        
        return new RedirectResponse($this->router->generate('app_home'));
    }

    public function onAuthenticationFailure(Request $request, AuthenticationException $exception): ?Response
    {
        $this->logger->error('OAuth authentication failure', [
            'message' => $exception->getMessage(),
            'code' => $exception->getCode()
        ]);
        
        return new JsonResponse([
            'error' => 'Authentication failed',
            'message' => $exception->getMessage(),
            'query_params' => $request->query->all()
        ], Response::HTTP_UNAUTHORIZED);
    }

    public function start(Request $request, AuthenticationException $authException = null): Response
    {
        return new RedirectResponse(
            $this->router->generate('connect_sso_start'),
            Response::HTTP_TEMPORARY_REDIRECT
        );
    }

    private function getSudalysClient()
    {
        return $this->clientRegistry->getClient('sudalys');
    }
}