<?php

namespace App\Security;

use App\Entity\User;
use Doctrine\ORM\EntityManagerInterface;
use KnpU\OAuth2ClientBundle\Client\ClientRegistry;
use KnpU\OAuth2ClientBundle\Security\User\OAuthUserProvider as BaseOAuthUserProvider;
use Symfony\Component\Security\Core\User\UserInterface;

// TODO : delete this file
class OAuthUserProvider extends BaseOAuthUserProvider
{
    private EntityManagerInterface $entityManager;
    private ClientRegistry $clientRegistry;
    
    public function __construct(EntityManagerInterface $entityManager, ClientRegistry $clientRegistry)
    {
        $this->entityManager = $entityManager;
        $this->clientRegistry = $clientRegistry;
    }
    
    /**
     * Loads a user from the given token.
     */
    public function loadUserByIdentifier(string $identifier): UserInterface
    {
        // Try to find an existing user with this identifier
        $userRepository = $this->entityManager->getRepository(User::class);
        $user = $userRepository->findOneBy(['username' => $identifier]);
        
        // If the user doesn't exist, create one
        if (!$user) {
            $user = new User();
            $user->setUsername($identifier);
            $user->setPassword(''); // No password for OAuth users
            
            // Try to get an email from the current client's token data if available
            try {
                $client = $this->clientRegistry->getClient('sso_server');
                $accessToken = $client->getAccessToken();
                if ($accessToken) {
                    $resourceOwner = $client->fetchUserFromToken($accessToken);
                    $userData = $resourceOwner->toArray();
                    
                    // Set email if available in the response
                    if (isset($userData['email'])) {
                        $user->setEmail($userData['email']);
                    } else {
                        $user->setEmail($identifier . '@example.com');
                    }
                } else {
                    $user->setEmail($identifier . '@example.com');
                }
            } catch (\Exception $e) {
                // Fallback if we can't get the email
                $user->setEmail($identifier . '@example.com');
            }
            
            $user->setRoles(['ROLE_USER']);
            
            $this->entityManager->persist($user);
            $this->entityManager->flush();
        }
        
        return $user;
    }
    
    /**
     * Check if this provider supports the given user class.
     */
    public function supportsClass($class): bool
    {
        return $class === User::class || is_subclass_of($class, User::class);
    }
} 